/**
 * 
 */
package fr.ill.ics.n3d.io;

import java.util.HashMap;

import org.w3c.dom.Element;

import fr.ill.ics.n3d.model.Component;
import fr.ill.ics.n3d.model.ConfigParams;
import fr.ill.ics.n3d.model.Model;

/**
 * @author dages
 *
 */
public class NameConverter {
	
	/** Mapping between old file names (keys) and new file names (values). */
	private HashMap<String, String> fileNameConversion;
	
	/** Mapping between old names (keys) and new names (values). */
	private HashMap<String, String> nameConversion;

	/**
	 * Default constructor.
	 */
	public NameConverter() {
		this.fileNameConversion = new HashMap<>();
		this.nameConversion = new HashMap<>();
	}
	
	public void convert(Model model) {
		this.fileNameConversion.clear();
		this.nameConversion.clear();
		
		AsciiStringConverter ascii = new AsciiStringConverter();
		model.setName(ascii.convert(model.getName()));
		convert(model.getRoot(), ascii);
	}
	
	private void convert(Component component, AsciiStringConverter ascii) {
		
		// Change the filename and the name.
		String convertedFileName = ascii.convert(component.getFileName());
		String newFileName = new String(convertedFileName);

		String convertedName = ascii.convert(component.getName());
		String newName = new String(convertedName);
		
		this.fileNameConversion.put(component.getFileName(), newFileName);
		component.setFileName(newFileName);
		this.nameConversion.put(component.getName(), newName);
		component.setName(newName);
		
		// Change the name of the configurations.
		for (ConfigParams config : component.getConfigurations()) {
			config.setConfiguration(ascii.convert(config.getConfiguration()));
		}
		
		// Iterate the children.
		for (Component child : component.getChildren()) {
			convert(child, ascii);
		}
	}
	
	public String newFileName(String oldFileName) {
		return this.fileNameConversion.get(oldFileName);
	}
	
	public String newName(String oldName) {
		return this.nameConversion.get(oldName);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "NameConverter [\nfileNameConversion=" + fileNameConversion + ",\n nameConversion=" + nameConversion + "\n]";
	}

}
