/**
 * 
 */
package fr.ill.ics.n3d.model;

import java.util.logging.Logger;

import javafx.geometry.Point3D;
import javafx.scene.transform.Transform;

/**
 * @author dages
 *
 */
public class CoincidentMate extends Mate {
	
	public static final String XML_TYPE_NAME = "Coincident";

	/**
	 * Default constructor.
	 */
	public CoincidentMate() {
		super();
	}

	@Override
	public void apply(Axis translation, Axis rotation, Transform rootToScene) {
		// Each mates reduce the number of DOFs
		Point3D rootOrigin = rootToScene.transform(Point3D.ZERO);
		Point3D[] matePositions = {
				rootToScene.transform(this.getEntity(0).getPosition()), 
				rootToScene.transform(this.getEntity(1).getPosition())
			};
		Point3D[] mateAxes = {
				rootToScene.transform(this.getEntity(0).getAxis()).subtract(rootOrigin).normalize(), 
				rootToScene.transform(this.getEntity(1).getAxis()).subtract(rootOrigin).normalize()
			};
		Point3D rotationPivot = Point3D.ZERO;
		
		switch (this.getEntity(0).getType()) {
			case CYLINDER:
			case LINE:
			case POINT:
				switch (this.getEntity(1).getType()) {
					case CYLINDER:
					case LINE:
					case POINT:
						rotationPivot = (this.getEntity(0).getRadius(0) > this.getEntity(1).getRadius(0)) ? matePositions[0] : matePositions[1];
						break;
					case PLANE:
						rotationPivot = matePositions[0];
						break;
					default:
						break;
				}
				break;
			case PLANE:
				switch(this.getEntity(1).getType()) {
					case CYLINDER:
					case LINE:
					case POINT:
						rotationPivot = matePositions[1];
						break;
					case PLANE:
						rotationPivot = matePositions[0].midpoint(matePositions[1]);
						break;
					default:
						break;
				}
				break;
			default:
				break;
		}
		
		switch (this.getEntity(0).getType()) {
			case CYLINDER:
			case LINE:
			case POINT:
				switch (this.getEntity(1).getType()) {
					case CYLINDER:
					case LINE:
					case POINT:
					case PLANE:
						if (mateAxes[0].crossProduct(mateAxes[1]).magnitude() < EPSILON) {
							if (translation.getType() == Axis.Type.NONE) {
								translation.setType(Axis.Type.TRANSLATION);
								translation.setDirection(mateAxes[0]);
							} else if (translation.getType() == Axis.Type.TRANSLATION) {
								if (mateAxes[0].crossProduct(translation.getDirection()).magnitude() > EPSILON) {
									translation.setDirection(Point3D.ZERO);
									translation.setType(Axis.Type.FIXED);
								}
							}

							if (mateAxes[0].crossProduct(matePositions[0].subtract(matePositions[1])).magnitude() > EPSILON) {
								rotation.setDirection(Point3D.ZERO);
								rotation.setPosition(rotationPivot);
								rotation.setType(Axis.Type.FIXED);
							} else {
								if (rotation.getType() == Axis.Type.NONE) {
									rotation.setType(Axis.Type.ROTATION);
									rotation.setDirection(mateAxes[0]);
									rotation.setPosition(rotationPivot);
								} else if (rotation.getType() == Axis.Type.ROTATION && mateAxes[0].crossProduct(rotation.getDirection()).magnitude() > EPSILON) {
									rotation.setType(Axis.Type.FIXED);
									rotation.setDirection(Point3D.ZERO);
									rotation.setPosition(rotationPivot);
								}
							}
						} else {
							Logger.getLogger("nomad-3d").severe("Mate axes non colinear for " + this.getName() + " (Coincident - " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + ")");
						}
						break;
					default:
						Logger.getLogger("nomad-3d").warning("Coincident mate entities " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + " not handled");
						break;
				}
				break;
			case PLANE:
				switch(this.getEntity(1).getType()) {
					case CYLINDER:
					case LINE:
					case POINT:
						if (mateAxes[0].crossProduct(mateAxes[1]).magnitude() < EPSILON) {
							if (translation.getType() == Axis.Type.NONE) {
								translation.setType(Axis.Type.TRANSLATION);
								translation.setDirection(mateAxes[0]);
							} else if (translation.getType() == Axis.Type.TRANSLATION) {
								if (mateAxes[0].crossProduct(translation.getDirection()).magnitude() > EPSILON) {
									translation.setDirection(Point3D.ZERO);
									translation.setType(Axis.Type.FIXED);
								}
							}

							if (rotation.getType() == Axis.Type.NONE) {
								rotation.setType(Axis.Type.ROTATION);
								rotation.setDirection(mateAxes[0]);
								rotation.setPosition(rotationPivot);
							} else if (rotation.getType() == Axis.Type.ROTATION && mateAxes[0].crossProduct(rotation.getDirection()).magnitude() > EPSILON) {
								rotation.setType(Axis.Type.FIXED);
								rotation.setDirection(Point3D.ZERO);
								rotation.setPosition(rotationPivot);
							}
						} else {
							Logger.getLogger("nomad-3d").severe("Mate axes non colinear for " + this.getName() + " (Coincident - " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + ")");
						}
						break;
					case PLANE:
						if (mateAxes[0].crossProduct(mateAxes[1]).magnitude() < EPSILON) {
							translation.setDirection(Point3D.ZERO);
							translation.setType(Axis.Type.FIXED);

							if (rotation.getType() == Axis.Type.NONE) {
								rotation.setType(Axis.Type.ROTATION);
								rotation.setDirection(mateAxes[0]);
								rotation.setPosition(rotationPivot);
							} else if (rotation.getType() == Axis.Type.ROTATION && mateAxes[0].crossProduct(rotation.getDirection()).magnitude() > EPSILON) {
								rotation.setType(Axis.Type.FIXED);
								rotation.setDirection(Point3D.ZERO);
								rotation.setPosition(rotationPivot);
							}
						} else {
							Logger.getLogger("nomad-3d").severe("Mate axes non colinear for " + this.getName() + " (Coincident - " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + ")");
						}
						break;
					default:
						Logger.getLogger("nomad-3d").warning("Coincident mate entities " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + " not handled");
						break;
				}
				break;
			default:
				Logger.getLogger("nomad-3d").warning("Coincident mate entities " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + " not handled");
				break;
		}
	}
	
	@Override
	public String toString() {
		return XML_TYPE_NAME + super.toString();
	}
}
