/**
 * 
 */
package fr.ill.ics.n3d.model;

import java.util.logging.Logger;

import javafx.geometry.Point3D;
import javafx.scene.transform.Transform;

/**
 * @author dages
 *
 */
public class PerpendicularMate extends Mate {
	
	public static final String XML_TYPE_NAME = "Perpendicular";

	/**
	 * Default constructor.
	 */
	public PerpendicularMate() {
		super();
	}

	@Override
	public void apply(Axis translation, Axis rotation, Transform rootToScene) {
		// Each mates reduce the number of DOFs
		Point3D rootOrigin = rootToScene.transform(Point3D.ZERO);
		Point3D[] matePositions = {rootToScene.transform(this.getEntity(0).getPosition()), rootToScene.transform(this.getEntity(1).getPosition())};
		Point3D[] mateAxes = {
				rootToScene.transform(this.getEntity(0).getAxis()).subtract(rootOrigin).normalize(), 
				rootToScene.transform(this.getEntity(1).getAxis()).subtract(rootOrigin).normalize()
			};
		
		switch (this.getEntity(0).getType()) {
			case PLANE:
				switch (this.getEntity(1).getType()) {
					case PLANE:
						if (Math.abs(mateAxes[0].dotProduct(mateAxes[1])) < EPSILON) {
							translation.setDirection(Point3D.ZERO);
							translation.setType(Axis.Type.FIXED);

							if (rotation.getType() == Axis.Type.NONE) {
								rotation.setType(Axis.Type.ROTATION);
								rotation.setDirection(mateAxes[0]);
								rotation.setPosition(matePositions[0].midpoint(matePositions[1]));
							} else if (rotation.getType() == Axis.Type.ROTATION 
									&& (Math.abs(mateAxes[0].dotProduct(rotation.getDirection())) > EPSILON 
											|| Math.abs(mateAxes[1].dotProduct(rotation.getDirection())) > EPSILON)) {
								rotation.setType(Axis.Type.FIXED);
								rotation.setDirection(Point3D.ZERO);
								rotation.setPosition(matePositions[0].midpoint(matePositions[1]));
							}
						} else {
							Logger.getLogger("nomad-3d").severe("Mate axes non perpendicular for " + this.getName() + " (" + "Perpendicular - " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + ")");
						}
						break;
					default:
						Logger.getLogger("nomad-3d").warning("Perpendicular mate entities " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + " not handled");
						break;
				}
				break;
			default:
				Logger.getLogger("nomad-3d").warning("Perpendicular mate entities " + this.getEntity(0).getType() + " - " + this.getEntity(1).getType() + " not handled");
				break;
		}
	}
	
	@Override
	public String toString() {
		return XML_TYPE_NAME + super.toString();
	}

}
